@@ -32,17 +32,15 @@
 
 #include <gck/gck.h>
 
-#include <egg/egg-bytes.h>
-
 G_BEGIN_DECLS
 
 typedef void             (*GcrOpensshPubCallback)          (GckAttributes *attrs,
                                                             const gchar *label,
                                                             const gchar *options,
-                                                            EggBytes *outer,
+                                                            GBytes *outer,
                                                             gpointer user_data);
 
-guint                    _gcr_openssh_pub_parse            (EggBytes *data,
+guint                    _gcr_openssh_pub_parse            (GBytes *data,
                                                             GcrOpensshPubCallback callback,
                                                             gpointer user_data);
 
