@@ -67,6 +67,8 @@ struct _GcrPromptIface {
 	GcrPromptReply     (* prompt_confirm_finish)    (GcrPrompt *prompt,
 	                                                 GAsyncResult *result,
 	                                                 GError **error);
+
+	void               (* prompt_close)             (GcrPrompt *prompt);
 };
 
 GType                gcr_prompt_get_type                  (void);
@@ -159,6 +161,8 @@ GcrPromptReply       gcr_prompt_confirm_run               (GcrPrompt *prompt,
                                                            GCancellable *cancellable,
                                                            GError **error);
 
+void                 gcr_prompt_close                     (GcrPrompt *prompt);
+
 G_END_DECLS
 
 #endif /* __GCR_PROMPT_H__ */
